library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity sha256transform is
   port 
   (
      clk       : in  std_logic;
      rx_state  : in  std_logic_vector(255 downto 0);
      rx_input  : in  std_logic_vector(511 downto 0);
      tx_hash   : out std_logic_vector(255 downto 0);
      valid     : out std_logic := '0'
   );
end sha256transform;

architecture arch of sha256transform is
  
   type t_k_para is array(0 to 63) of std_logic_vector(31 downto 0);
   constant Ks : t_k_para := (
      x"428a2f98", x"71374491", x"b5c0fbcf", x"e9b5dba5", 
      x"3956c25b", x"59f111f1", x"923f82a4", x"ab1c5ed5", 
      x"d807aa98", x"12835b01", x"243185be", x"550c7dc3", 
      x"72be5d74", x"80deb1fe", x"9bdc06a7", x"c19bf174", 
      x"e49b69c1", x"efbe4786", x"0fc19dc6", x"240ca1cc", 
      x"2de92c6f", x"4a7484aa", x"5cb0a9dc", x"76f988da", 
      x"983e5152", x"a831c66d", x"b00327c8", x"bf597fc7", 
      x"c6e00bf3", x"d5a79147", x"06ca6351", x"14292967", 
      x"27b70a85", x"2e1b2138", x"4d2c6dfc", x"53380d13", 
      x"650a7354", x"766a0abb", x"81c2c92e", x"92722c85", 
      x"a2bfe8a1", x"a81a664b", x"c24b8b70", x"c76c51a3", 
      x"d192e819", x"d6990624", x"f40e3585", x"106aa070", 
      x"19a4c116", x"1e376c08", x"2748774c", x"34b0bcb5", 
      x"391c0cb3", x"4ed8aa4a", x"5b9cca4f", x"682e6ff3", 
      x"748f82ee", x"78a5636f", x"84c87814", x"8cc70208", 
      x"90befffa", x"a4506ceb", x"bef9a3f7", x"c67178f2"
   );
   
   signal w_in      : std_logic_vector(511 downto 0);
   signal state_in  : std_logic_vector(255 downto 0);

   signal w_out     : std_logic_vector(511 downto 0);
   signal state_out : std_logic_vector(255 downto 0);
   
   signal step : unsigned(5 downto 0) := (others => '0');
   
   signal k_in        : std_logic_vector(31 downto 0) := x"428a2f98";
   
   signal first_mux : std_logic := '1';
   
begin

   w_in     <= rx_input when first_mux='1' else w_out;
   state_in <= rx_state when first_mux='1' else state_out;
   
   isha256digester : entity work.sha256digester
      port map(clk, w_in, state_in, w_out, state_out, k_in);
  
   
   process (clk)
   begin
      if rising_edge(clk) then
      
         k_in <= Ks(to_integer(step+1));
      
         if (step = 0) then
            valid <= '1';
         else
            valid <= '0';
         end if;
         
         if (step = 63) then
            first_mux <= '1';
         else
            first_mux <= '0';
         end if;
      
         step <= step + 1;
      
         tx_hash(31  downto 0  ) <= std_logic_vector(unsigned(rx_state(31  downto 0  )) + unsigned(state_out(31  downto 0  )));
         tx_hash(63  downto 32 ) <= std_logic_vector(unsigned(rx_state(63  downto 32 )) + unsigned(state_out(63  downto 32 )));
         tx_hash(95  downto 64 ) <= std_logic_vector(unsigned(rx_state(95  downto 64 )) + unsigned(state_out(95  downto 64 )));
         tx_hash(127 downto 96 ) <= std_logic_vector(unsigned(rx_state(127 downto 96 )) + unsigned(state_out(127 downto 96 )));
         tx_hash(159 downto 128) <= std_logic_vector(unsigned(rx_state(159 downto 128)) + unsigned(state_out(159 downto 128)));
         tx_hash(191 downto 160) <= std_logic_vector(unsigned(rx_state(191 downto 160)) + unsigned(state_out(191 downto 160)));
         tx_hash(223 downto 192) <= std_logic_vector(unsigned(rx_state(223 downto 192)) + unsigned(state_out(223 downto 192)));
         tx_hash(255 downto 224) <= std_logic_vector(unsigned(rx_state(255 downto 224)) + unsigned(state_out(255 downto 224)));
         
      end if;
   end process;
   
  
end architecture;